@@ -42,6 +42,7 @@ static gchar*       get_uri_from_text                            (HtmlBoxText
 static void         html_link_accessible_action_interface_init   (AtkActionIface    *iface);
 static gboolean     html_link_accessible_do_action               (AtkAction         *action,
                                                                   gint              i);
+gboolean            idle_do_action                               (gpointer          data);
 static gint         html_link_accessible_get_n_actions           (AtkAction         *action);
 
 static G_CONST_RETURN gchar*   html_link_accessible_get_description  (AtkAction         *action,
@@ -114,6 +115,11 @@ html_link_accessible_finalize (GObject *object)
 		g_object_remove_weak_pointer (G_OBJECT (link->obj),
 					      (gpointer *)&link->obj);
 	g_free (link->click_description);
+	if (link->action_idle_handler) {
+		gtk_idle_remove (link->action_idle_handler);
+		link->action_idle_handler = 0;
+	}
+
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -201,10 +207,10 @@ get_box_text_from_accessible (AtkObject *obj)
 	AtkGObjectAccessible *atk_gobj;
 	GObject *g_obj;
 
-	g_return_if_fail (ATK_IS_GOBJECT_ACCESSIBLE (obj));
+	g_return_val_if_fail (ATK_IS_GOBJECT_ACCESSIBLE (obj), NULL);
 	atk_gobj  = ATK_GOBJECT_ACCESSIBLE (obj);
 	g_obj = atk_gobject_accessible_get_object (atk_gobj);
-	g_return_if_fail (HTML_IS_BOX_TEXT (g_obj));
+	g_return_val_if_fail (HTML_IS_BOX_TEXT (g_obj), NULL);
 	return HTML_BOX_TEXT (g_obj);
 }
 
@@ -248,27 +254,43 @@ static gboolean
 html_link_accessible_do_action (AtkAction *action,
                                 gint       i)
 {
+	HtmlLinkAccessible *link;
+
 	if (i == 0) {
-		HtmlLinkAccessible *link;
-
-		HtmlBoxText *text;
-		GtkWidget *view;
-		gchar *uri;
-
 		link = HTML_LINK_ACCESSIBLE (action);
-		text = get_box_text_from_accessible (link->obj);
-		view = html_box_accessible_get_view_widget (HTML_BOX (text));
-		uri = get_uri_from_text (text);
-		g_signal_emit_by_name (HTML_VIEW (view)->document,
-				       "link_clicked",
-				       uri);
-		g_free (uri);
-		return TRUE;
+		if (link->action_idle_handler)
+			return FALSE;
+		else {
+			link->action_idle_handler = gtk_idle_add (idle_do_action, link);
+			return TRUE;
+		}
 	} else {
 		return FALSE;
 	}
 }
 
+gboolean
+idle_do_action (gpointer data)
+{
+	HtmlLinkAccessible *link;
+
+	HtmlBoxText *text;
+	GtkWidget *view;
+	gchar *uri;
+
+	link = HTML_LINK_ACCESSIBLE (data);
+	link->action_idle_handler = 0;
+	text = get_box_text_from_accessible (link->obj);
+	view = html_box_accessible_get_view_widget (HTML_BOX (text));
+	uri = get_uri_from_text (text);
+	g_signal_emit_by_name (HTML_VIEW (view)->document,
+			       "link_clicked",
+			       uri);
+	g_free (uri);
+	return FALSE;
+}
+
+
 static gint
 html_link_accessible_get_n_actions (AtkAction *action)
 {
